# mostly taken from https://scikit-learn.org/stable/auto_examples/cluster/plot_kmeans_assumptions.html

from sklearn.cluster import KMeans
from sklearn.datasets import make_blobs
import matplotlib.pyplot as plt
import numpy as np

X_a, y_a = make_blobs(n_samples=1500, random_state=170)
X_b, y_b = np.dot(X_a, [[0.60834549, -0.63667341], [-0.40887718, 0.85253229]]), y_a
X_c, y_c = make_blobs(n_samples=1500, cluster_std=[1.0, 2.5, 0.5], random_state=170)
X_d, y_d = np.vstack((X_a[y_a == 0][:500], X_a[y_a == 1][:100], X_a[y_a == 2][:10])), [0] * 500 + [1] * 100 + [2] * 10

# Plot Data
fig, axs = plt.subplots(nrows=2, ncols=2, figsize=(8, 8))
axs[0, 0].scatter(X_a[:, 0], X_a[:, 1], c=y_a)
axs[0, 1].scatter(X_b[:, 0], X_b[:, 1], c=y_b)
axs[1, 0].scatter(X_c[:, 0], X_c[:, 1], c=y_c)
axs[1, 1].scatter(X_d[:, 0], X_d[:, 1], c=y_d)

km_a = KMeans(n_clusters=2, random_state=170).fit(X_a)
km_b = KMeans(n_clusters=3, random_state=170).fit(X_b)
km_c = KMeans(n_clusters=3, random_state=170).fit(X_c)
km_d = KMeans(n_clusters=3, random_state=170).fit(X_d)

# Plot Cluster
fig, axs = plt.subplots(nrows=2, ncols=2, figsize=(8, 8))
axs[0, 0].scatter(X_a[:, 0], X_a[:, 1], c=km_a.labels_)
axs[0, 0].scatter(km_a.cluster_centers_[:, 0], km_a.cluster_centers_[:, 1], marker="d", c="red")
axs[0, 1].scatter(X_b[:, 0], X_b[:, 1], c=km_b.labels_)
axs[0, 1].scatter(km_b.cluster_centers_[:, 0], km_b.cluster_centers_[:, 1], marker="d", c="red")
axs[1, 0].scatter(X_c[:, 0], X_c[:, 1], c=km_c.labels_)
axs[1, 0].scatter(km_c.cluster_centers_[:, 0], km_c.cluster_centers_[:, 1], marker="d", c="red")
axs[1, 1].scatter(X_d[:, 0], X_d[:, 1], c=km_d.labels_)
axs[1, 1].scatter(km_d.cluster_centers_[:, 0], km_d.cluster_centers_[:, 1], marker="d", c="red")


plt.show()