import numpy as np

# data here given as python lists and array, would
#	be read from database or files in reality
ATTRIBUTES = [
	"gewicht",
	"haltbarkeit",
	"preis"
] 
ARTICLES = {
	"A" : {
		"name" : "Aepfel",
		"gewicht" : 6,
		"haltbarkeit" : 4,
		"preis" : 1
	},
	"B" : {
		"name" : "Brot",
		"gewicht" : 2,
		"haltbarkeit" : 1,
		"preis" : 8
	},
	"H" : {
		"name" : "Haferflocken",
		"gewicht" : 4,
		"haltbarkeit" : 4,
		"preis" : 6
	},
	"K" : {
		"name" : "Kartoffeln",
		"gewicht" : 9,
		"haltbarkeit" : 4,
		"preis" : 1
	},
	"Z" : {
		"name" : "Zucker",
		"gewicht" : 1,
		"haltbarkeit" : 10,
		"preis" : 9
	}
}

TRANSACTIONS = [
	(1, ["B", "H", "K"]),
	(2, ["A", "B", "Z"]),
	(3, ["A", "B", "K", "Z"]),
	(2, ["B", "K", "Z"]),
	(4, ["B", "H", "K", "Z"])
]

# create the content vector
def content(s:str) -> np.ndarray:
	return np.array([ ARTICLES[s][attribute] for attribute in ATTRIBUTES])

print(content("A"))

# create the profile array for each user
#	should not be done each time, better store user profiles 
#	as NumPy Objects
def profile(c:int) -> np.ndarray:
	v, n = np.zeros(len(ATTRIBUTES)), 0
	for user, articles in TRANSACTIONS:
		if c == user:
			n += len(articles)
			for article in articles:
				v += content(article)

	return v/n

print(profile(1))

# 	should be implemented to fetch dynamically all articles and users in reality (instead of hardcoded values)
contents = np.vstack((content("A"), content("B"), content("H"), content("K"), content("Z")))
profiles = np.vstack((profile(1), profile(2), profile(3), profile(4)))
print(contents, profiles)

from sklearn.metrics.pairwise import cosine_similarity

# utility of each article for each user
#		  A  B  H  K  Z
#	user 1
#	user 2
print(cosine_similarity(profiles, contents))

def new_article(profiles, article_content):
	utilities = cosine_similarity(profiles, article_content)
	print(utilities)

	best = np.argmax(utilities)
	print("Recommend", article_content, "to User", best+1)
	
	return best

new_article(profiles, [[5, 2, 1]])