
import numpy as np
from sklearn.datasets import fetch_20newsgroups
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.cluster import KMeans

# train model
X_train, y_train = fetch_20newsgroups(
	remove=["headers", "footers", "quotes"],
	categories=["comp.graphics", "rec.autos"],
	subset="train",
	return_X_y=True
)
num_labels = np.unique(y_train).shape[0]

tfidf = TfidfVectorizer(stop_words="english")
tfidf.fit(X_train)

kmeans = KMeans(n_clusters=num_labels, n_init="auto", random_state=42)
kmeans.fit(tfidf.transform(X_train))

# test model
from sklearn.metrics import accuracy_score

X_test, y_test = fetch_20newsgroups(
	remove=["headers", "footers", "quotes"],
	categories=["comp.graphics", "rec.autos"],
	subset="test",
	return_X_y=True
)
y_prediction = kmeans.predict(tfidf.transform(X_test))

print(np.unique(y_test == y_prediction, return_counts=True))

print(accuracy_score(y_test, y_prediction))