
import numpy as np
from sklearn.datasets import fetch_20newsgroups
from sklearn.feature_extraction.text import TfidfVectorizer

# data
X_train, y_train = fetch_20newsgroups(
	remove=["headers", "footers", "quotes"],
	categories=["comp.graphics", "rec.autos"],
	subset="train",
	return_X_y=True
)
X_test, y_test = fetch_20newsgroups(
	remove=["headers", "footers", "quotes"],
	categories=["comp.graphics", "rec.autos"],
	subset="test",
	return_X_y=True
)
num_labels = np.unique(y_train).shape[0]

# model
tfidf = TfidfVectorizer(stop_words="english")
X_train_tfidf = tfidf.fit_transform(X_train)

# predict (fetch similar document)
from sklearn.metrics.pairwise import cosine_similarity
from collections import Counter

def fetch_similar_documents(doc:str, top_n:int=10):
	sim = cosine_similarity(tfidf.transform([doc]), X_train_tfidf)
	return (-sim).argsort(axis=None)[:top_n]
	
for i in [2, 10, 20, 30, 50, 100, 400]:
	best = fetch_similar_documents(X_test[i])
	y_counts = Counter([y_train[b] for b in best])
	print("Document {: >3} (class {}) fetched {: >2} times same class: {}".format(
		i, y_test[i], y_counts[y_test[i]], best
	))

