
import torch
from sklearn.datasets import fetch_20newsgroups

class NewsgroupsData(torch.utils.data.Dataset):
	def __init__(self, tokenizer, train=True, reduce_size=True):
		self.tokenizer = tokenizer

		self.X, self.y = fetch_20newsgroups(
			remove=["headers", "footers", "quotes"],
			subset="train" if train else "test",
			return_X_y=True
		)
		if reduce_size:
			self.X, self.y = self.X[:50], self.y[:50]

	def __getitem__(self, index):
		item = self.tokenizer(self.X[index],
				truncation=True,
				padding="max_length"
			)
		item['labels'] = self.y[index]
		return item
		
	def __len__(self):
		return len(self.X)


import numpy as np
from sklearn.metrics import accuracy_score

def get_accuracy(eval_result):
	logits, labels = eval_result
	predicts = np.argmax(logits, axis=-1)
	return { "accuracy" : accuracy_score(labels, predicts) }

from transformers import (
	Trainer, TrainingArguments,
	BertTokenizer, BertForSequenceClassification
)

model = BertForSequenceClassification.from_pretrained("bert-base-uncased", num_labels=20)
tokenizer = BertTokenizer.from_pretrained("bert-base-uncased")
trainer = Trainer(
	model=model,
	args=TrainingArguments(
		output_dir="test_trainer",
		overwrite_output_dir=True
	),
	train_dataset=NewsgroupsData(tokenizer, train=True),
	eval_dataset=NewsgroupsData(tokenizer, train=False),
	compute_metrics=get_accuracy
)

trainer.train()
trainer.evaluate()

# trainer.save_model("./my-model")
# BertForSequenceClassification.from_pretrained("./my-model")